/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;

	
namespace ShopClassLibrary
{
	public interface ISaleTeste
	{
	    DataSet GetById(string User, string Pass, int saleid);
        
        ShopStatusEnum GenerateWordDocumentInvoice(string User, string Pass, int saleid, out string filename);
    }

	/// <summary>
	/// Summary description for Sale.
	/// </summary>
	public class SaleTesteImpl : ISaleTeste
    {
        private const string OUTDIR_DOC = @"c:\temp\";

        public DataSet GetById(string User, string Pass, int saleid)
        {
            DataSet ds = null;
            OleDbConnection conn = null;

            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar objecto de conexo  base de dados e abrir a conexo
                //
                conn = new OleDbConnection(UtilDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                if (UtilDB.ValidateUser(conn, null, User, Pass) != ShopStatusEnum.OK)
                    return null;

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                // cabealhos das vendas
                //
                string sSqlCmd = "SELECT * FROM vwTotalBySale WHERE SaleId=?";

                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                
                oAdapter.SelectCommand.Parameters.AddWithValue("sid", saleid);

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, "Sales");

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                // linhas de detalhe
                //
                sSqlCmd = "SELECT * FROM vwExpandedSaleDetails WHERE SaleId=?";

                oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.AddWithValue("sid", saleid);

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, "SaleDetails");

                //
                // criar relao entre tabelas
                //
                ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepo!!!!
                //
                System.Console.WriteLine("EXCEPO no mtodo 'GetById': " + ex);
                ds = null;
            }
            finally
            {
                //
                // fechar a conexo
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

        public ShopStatusEnum GenerateWordDocumentInvoice(string User, string Pass, int saleid, out string filename)
        {
            object missingValue = Type.Missing;

            object uchar = Microsoft.Office.Interop.Word.WdUnits.wdCharacter;
            object uline = Microsoft.Office.Interop.Word.WdUnits.wdLine;
            object ucol = Microsoft.Office.Interop.Word.WdUnits.wdColumn;
            object i1 = 1;
            object i4 = 4;
            object bfalse = false;
            object btrue = true;

            filename = null;

            // obter factura
            DataSet dsFactura = GetById(User, Pass, saleid);
            if (dsFactura == null)
                return ShopStatusEnum.ERROR;
            
            int nFactura = saleid; //devia existir um campo separado para o numero de factura que tem que ser sequencial e sem "buracos";
            DateTime dataFactura = (DateTime)dsFactura.Tables["Sales"].Rows[0]["DateOfSale"];
            string customerName = (string)dsFactura.Tables["Sales"].Rows[0]["Name"];
            double totalFactura = (double)dsFactura.Tables["Sales"].Rows[0]["SaleTotal"];

            // gerar nome de ficheiro
            object _in_filename = OUTDIR_DOC + "Factura-" + nFactura.ToString() + ".doc";

            // criar objecto Word
            Microsoft.Office.Interop.Word.Application w = new Microsoft.Office.Interop.Word.ApplicationClass();

            // criar novo documento
            w.Documents.Add(ref missingValue, ref missingValue, ref missingValue, ref missingValue);

            //w.Visible = true;	// s para teste/"debug"!!!!!

            w.Selection.TypeText("Factura n ");
            w.Selection.TypeText(nFactura.ToString());
            w.Selection.TypeParagraph();
            w.Selection.TypeText("Data : ");
            w.Selection.TypeText(dataFactura.ToLongDateString());
            w.Selection.TypeParagraph();
            w.Selection.TypeText("Cliente : ");
            w.Selection.TypeText(customerName);
            w.Selection.TypeParagraph();
            w.Selection.TypeParagraph();

            //
            // criar tabela (cabealho)
            //
            object tb = Microsoft.Office.Interop.Word.WdDefaultTableBehavior.wdWord9TableBehavior;
            object afb = Microsoft.Office.Interop.Word.WdAutoFitBehavior.wdAutoFitWindow;
            w.ActiveDocument.Tables.Add(w.Selection.Range, 1, 4, ref tb, ref afb);

            w.Selection.TypeText("Quantidade");
            w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            w.Selection.TypeText("Designao");
            w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            w.Selection.TypeText("P.U:");
            w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            w.Selection.TypeText("sub-total");

            //
            // adicionar as linhas de factura
            //
            foreach (DataRow dr in dsFactura.Tables["SaleDetails"].Rows)
            {
                // adicionar linha  tabela
                w.ActiveDocument.Tables[1].Rows.Add(ref missingValue);
                w.Selection.MoveDown(ref uline, ref i1, ref missingValue);
                w.Selection.MoveLeft(ref uchar, ref i4, ref missingValue);

                w.Selection.TypeText(dr["Quantity"].ToString());
                w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
                w.Selection.TypeText(dr["Description"].ToString());
                w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
                w.Selection.TypeText(dr["UnitPrice"].ToString());
                w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
                w.Selection.TypeText(dr["TotalCost"].ToString());
            }
            w.Selection.MoveDown(ref uline, ref i1, ref missingValue);

            w.Selection.TypeParagraph();
            w.Selection.TypeText("Total :");
            w.Selection.TypeText(totalFactura.ToString());
            w.Selection.TypeParagraph();

            //
            // formatar tabela
            //
            object tf = Microsoft.Office.Interop.Word.WdTableFormat.wdTableFormatProfessional;
            w.ActiveDocument.Tables[1].AutoFormat(ref tf, ref btrue, ref btrue, ref btrue, ref btrue, ref btrue, ref bfalse, ref bfalse, ref bfalse, ref bfalse);

            // rodap da factura
            w.Selection.TypeParagraph();
            w.Selection.TypeText("Gerada por ADAV 2006/2007");

            // gravar ficheiro
            w.ActiveDocument.SaveAs(ref _in_filename, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue);

            //terminar Word
            w.Quit(ref missingValue, ref missingValue, ref missingValue);

            filename = (string)_in_filename;
            
            return ShopStatusEnum.OK;
        }  
	}
}
